@@ -322,7 +322,7 @@ class MemorizingTransformer(nn.Module):
         ff_mult = 4,
         ff_dropout = 0.,
         memorizing_layers = None,
-        max_knn_memories = 2048,
+        max_knn_memories = 250000,
         num_retrieved_memories = 32,
         clear_memories_on_sos_token_id = None,
         knn_memories_directory = DEFAULT_KNN_MEMORY_MEMMAP_DIRECTORY,
@@ -332,8 +332,7 @@ class MemorizingTransformer(nn.Module):
         intra_attn_values_gating = False,
         xl_max_memories = 0,
         xl_memory_layers = None,
-        shift_xl_memories_down = 0.,
-        memory_expiration_fn = None
+        shift_xl_memories_down = 0.
     ):
         super().__init__()
         self.token_emb = nn.Embedding(num_tokens, dim)
@@ -367,7 +366,6 @@ class MemorizingTransformer(nn.Module):
         self.num_memory_layers = len(memorizing_layers)
 
         self.knn_use_gpu = knn_use_gpu
-        self.memory_expiration_fn = memory_expiration_fn
         self.clear_memories_on_sos_token_id = clear_memories_on_sos_token_id
 
         # relative positional bias
@@ -416,7 +414,7 @@ class MemorizingTransformer(nn.Module):
         knn_memories_directory = default(knn_memories_directory, self.knn_memories_directory)
         memories_dir = Path(knn_memories_directory)
 
-        return [KNNMemory(dim = self.dim_head, max_memories = self.max_knn_memories, knn_use_gpu = self.knn_use_gpu, num_indices = batch_size, expire_memory_fn = self.memory_expiration_fn, memmap_filename = str(memories_dir / f'knn.memory.layer.{ind + 1}.memmap')) for ind in range(self.num_memory_layers)]
+        return [KNNMemory(dim = self.dim_head, max_memories = self.max_knn_memories, knn_use_gpu = self.knn_use_gpu, num_indices = batch_size, memmap_filename = str(memories_dir / f'knn.memory.layer.{ind + 1}.memmap')) for ind in range(self.num_memory_layers)]
 
     @contextmanager
     def knn_memories_context(
